part of worksdata;

enum WorksActionType {
  BUTTON,
  START_PAGE,
  END_PAGE,
  EVENT,
  DYNAMIC_LINK,
  PUSH_NOTIFICATION,
  TAB,
  PAUSED_PAGE,
  RESUMED_PAGE,
}

class Worksdata {
  static final Worksdata _worksdata = Worksdata._internal();

  factory Worksdata() {
    return _worksdata;
  }

  Worksdata._internal();

  final Uuid _uuid = Uuid();
  final String _platform = Platform.operatingSystem.toUpperCase();
  final _api = ApiService(api: API.worksdata());
  String _tempId;

  /// To initiate Works plugin. Ensure this method is called in the main.dart or
  /// the place initiating the application.
  ///
  void init() {
    _setTempId();
    log('SYSTEM_INIT', worksActionType: WorksActionType.EVENT);
  }

  /// Send log information to Works API
  ///
  /// --actionId: log identifier to determine what log is this. (String with UPPER_SNAKE_CASE)
  /// --data: extra data to this log.
  /// --type: log type. eg: EVENT, BUTTON, DYNAMIC_LINK (Prefer using upper snake case)
  /// --worksActionType: replace 'type' if defined. Use predefined 'WorksActionType'.
  ///
  void log(String actionId,
      {Map data, String type, WorksActionType worksActionType}) {
    var actionLog = {
      'id': actionId.toUpperCase(),
      'type': worksActionType != null
          ? worksActionType.toString().split('.').last
          : (type ?? '').toUpperCase(),
      'created_time': DateTime.now().millisecondsSinceEpoch,
      'temp_id': _tempId,
      'platform': _platform,
      'data': data,
    };
    _api.post(APIPath.log(), body: actionLog);
  }

  /// Get uid & sending mapping between uid and tempId to Works API
  ///
  /// --uid: user's ID. Usually can be retrieved when logging in the user.
  /// --toLog: to log or not this event(login)
  ///
  void login(String uid, {bool toLog = true}) {
    if (toLog) log('SYSTEM_LOGIN', worksActionType: WorksActionType.EVENT);
    _api.get(APIPath.login(),
        queryParameters: {'uid': uid, 'temp_id': _tempId});
  }

  /// Flush uid & reset the tempId.
  ///
  /// --toLog: to log or not this event(logout)
  ///
  void logout({bool toLog = true}) {
    _setTempId();
    if (toLog) log('SYSTEM_LOGOUT', worksActionType: WorksActionType.EVENT);
  }

  void _setTempId() {
    _tempId = _getTempId();
  }

  String _getTempId() =>
      '${DateTime.now().millisecondsSinceEpoch.toString()}${_uuid.v4()}';
}
